require("Helpers")

CloverBagPathPoints = CloverBagPathPoints or {}

local ACTIVE_TIME = 0.6
local REST_POINT_ID = "RestPoint"
local ACTIVE_POINT_ID = "ActivePoint"
local PATH_POINT_COMPONENT_ID = "PathPointComponent"

function CloverBagPathPoints:construct()

	-- for storing the path points when we get them
	self.restPoint = nil
	self.activePoint = nil
	
	-- storing the spawn root
	self.spawnRoot = nil;
	
	-- sounds
	self.riseSound = nil;
	
	-- timer to keep this path point inactive when clovers are released
	self.activeTimer = 0.0
end

function CloverBagPathPoints:FirstUpdate()

	-- resolve the path points
	self:ResolvePathPoints()
	self:ResolveSpawnRoot()
	
	self.riseSound = GetGlobalSoundCue("EventStPatricksDayBagAppear")

	-- Will start inactive
	self:OnDeactive()
end

function CloverBagPathPoints:ResolvePathPoints()
	
	-- get the parent actor
	local parentActor = self:GetParentActor()
	if not parentActor then
		PrintString("Unable to find parent actor!")
		return
	end
	
	if not self.restPoint then
		-- Get the rest points
		local restPointActor = parentActor:FindEntity(REST_POINT_ID)
		if restPointActor then
			self.restPoint = restPointActor:FindEntity(PATH_POINT_COMPONENT_ID)
			if not self.restPoint then
				self:LogE("Unable to find " .. PATH_POINT_COMPONENT_ID .. " for " .. REST_POINT_ID)
			end
		else
			self:LogE("Unable to find " .. REST_POINT_ID)
		end
	end
	
	if not self.activePoint then
		-- Get the active points
		local activePointActor = parentActor:FindEntity(ACTIVE_POINT_ID)
		if activePointActor then
			self.activePoint = activePointActor:FindEntity(PATH_POINT_COMPONENT_ID)
			if not self.activePoint then
				self:LogE("Unable to find " .. PATH_POINT_COMPONENT_ID .. " for " .. ACTIVE_POINT_ID)
			end
		else
			self:LogE("Unable to find " .. ACTIVE_POINT_ID)
		end
	end
end

function CloverBagPathPoints:ResolveSpawnRoot()
	if self.spawnRoot then
		return
	end
	
	-- Getting the task this component is under
	local task = self:GetTask()
	if not task then
		self:LogE("Parent task is NULL!")
		return
	end
	
	-- Get the Actor that contains all the spawned clovers
	local spawnedCloversActor = task:FindEntity("SpawnedClovers")
	if not spawnedCloversActor then
		self:LogE("Unable to find 'SpawnedClovers'!")
		return
	end
	
		-- Get the SpawnRoot component attached to this Actor
	self.spawnRoot = spawnedCloversActor:FindEntity("SpawnRoot")
	if not self.spawnRoot then
		self:LogE("Unable to find 'SpawnRoot'!")
		return
	end
end

function CloverBagPathPoints:Update(dt)

	-- Only run if the game is not paused
	if Game.IsPaused() then
		return
	end
	
	if not self.spawnRoot then
		return
	end
	
	-- Get all the spawned clover Actors
	local spawnedCloverActors = self.spawnRoot:GetSpawnedActors()
	if not spawnedCloverActors then
		self:LogE("Spawned clover actors is nil!")
		return		
	end
	
	-- Check if any clover has its TouchDrag component captured
	local count = #spawnedCloverActors
	for i = 1, count do
		local actor = spawnedCloverActors[i]
		local touchDrag = actor:FindEntity("TouchDrag")
		if touchDrag and touchDrag:IsTouchCaptured() then
			-- a clover is being held down
			self:OnActivate()
			return
		end
	end
	
	if self.activeTimer > 0.0 then
		self.activeTimer = self.activeTimer - dt
		return
	end

	-- no clovers held down
	self:OnDeactive()
end

function CloverBagPathPoints:OnActivate()

	-- activate active path point
	if self.activePoint and not self.activePoint:IsEnabled() then
		self.activePoint:Enable()
	end
	
	-- deactivate rest path point
	if self.restPoint and self.restPoint:IsEnabled() then
		self.restPoint:Disable()
	end
	
	if self.activeTimer <= 0.0 and self.riseSound then
		self.riseSound:Play()
	end	
	
	self.activeTimer = ACTIVE_TIME
end

function CloverBagPathPoints:OnDeactive()

	-- deactivate active path point
	if self.activePoint and self.activePoint:IsEnabled() then
		self.activePoint:Disable()
	end
	
	-- activate rest path point
	if self.restPoint and not self.restPoint:IsEnabled() then
		self.restPoint:Enable()
	end

	if self.riseSound then
		self.riseSound:Stop()
	end
end