require("Helpers")

GhostAIComponent = GhostAIComponent or {}

local APPEAR = 0
local IDLE = 1
local CHASE = 2

function GhostAIComponent:construct()
	self.state = APPEAR
	
	-- APPEAR
	self.appearDuration = self:CreateProperty("Appear Duration", "float", 1.0)
	self.appearTimer = 0
	
	-- IDLE
	self.idleDuration = self:CreateProperty("Idle Duration", "float", 1.0)
	self.idleTimer = 0
	
	-- CHASE
	self.targetFruit = nil
	self.chaseSpeed = self:CreateProperty("Chase Speed", "float", 100.0)
	self.captureRadius = self:CreateProperty("Capture Radius", "float", 100.0)
	self.screenChasePadding = self:CreateProperty("Screen Chase Padding", "vector2", Vector2(-50,-50))
	self.resolvedArea = Vector2(100,100)
	self:ResolvePadding()
	
	self.disappearEmitter = nil
	self.spawnedEmitters = nil
	self.lastPos = nil
end

function GhostAIComponent:OnPropertyChanged(prop)
	if prop == self.screenChasePadding then
		self:ResolvePadding()
	end
end

function GhostAIComponent:OnDestroy()
	if not self.disappearEmitter or not self.spawnedEmitters or not self.lastPos then
		return
	end
	
	local newEmitter = self.disappearEmitter:Clone()
	if not newEmitter then
		self:LogE("Failed to clone disappear emitter!")
		return
	end
	
	self.spawnedEmitters:AddChild(newEmitter)
	SetProp(newEmitter, "position", "vector3", self.lastPos)
	SetProp(newEmitter, "enabled", "bool", true)
end

function GhostAIComponent:OnEnable()
	self:BeginAppear()
	
	local spawnSound = GetGlobalSoundCue("GhostbustersGhostSpawn")
	if not spawnSound then
		self:LogE("Missing sound GhostbustersGhostSpawn!")
	else
		spawnSound:Play()
	end
	
	self.disappearEmitter = self:FindEntity("parent.parent.parent.parent.DisappearEmitter")
	self.spawnedEmitters = self:FindEntity("parent.parent.parent.parent.SpawnedEmitters")
end

function GhostAIComponent:Update(dt)
	
	if Game.IsPaused() then
		return
	end
	
	local legacyDt = Game.GetLegacyDt()
	
	self.lastPos = self:GetPosition()
	
	if self.state == APPEAR then
		self:UpdateAppear(legacyDt)
	elseif self.state == IDLE then
		self:UpdateIdle(legacyDt)
	elseif self.state == CHASE then
		self:UpdateChaseFruit(legacyDt)
	else
		self:BeginIdle()
		self:UpdateIdle(legacyDt)
	end
end

--
-- APPEAR
--
function GhostAIComponent:BeginAppear()
	self.state = APPEAR
	self.appearTimer = 0
end

function GhostAIComponent:UpdateAppear(dt)
	self.appearTimer = self.appearTimer + dt
	
	if self.appearTimer >= self.appearDuration:GetValue() then
		self:EndAppear()
		return
	end
end

function GhostAIComponent:EndAppear()
	-- will always go to IDLE, can never go back to APPEAR
	self:BeginIdle()
end

--
-- IDLE
--
function GhostAIComponent:BeginIdle()
	self.state = IDLE
	self.idleTimer = 0
	local wanderer = self:GetSiblingComponentOfType("Animator_ScreenWander")
	if not wanderer then
		self:LogW("Missing Animator_ScreenWander!")
		return
	end
	
	wanderer:Enable()
end

function GhostAIComponent:UpdateIdle(dt)
	self.idleTimer = self.idleTimer + dt
	
	if self.idleTimer >= self.idleDuration:GetValue() then
		self:EndIdle()
		return
	end	
end

function GhostAIComponent:EndIdle()
	local wanderer = self:GetSiblingComponentOfType("Animator_ScreenWander")
	if wanderer then
		wanderer:Disable()
	end
	
	-- Enough waiting around, let's get a fruit!
	self:BeginChase()		
end

--
-- CHASE
--
function GhostAIComponent:BeginChase()
	self.state = CHASE
end

function GhostAIComponent:UpdateChaseFruit(dt)
	self:CheckTargetFruit()
	
	if not self.targetFruit then
		-- no fruit available, back to idle
		self:EndChase()
		-- switch to idle
		self:BeginIdle()
		return
	end
	
	-- have a target fruit, go after it!
	local position = self:GetPosition()
	local targetPosition = GetProp(self.targetFruit, "position", "vector3")
	local vec = targetPosition - position
	local dist = vec:MagnitudeSqr()
	
	if dist < self.captureRadius:GetValue() * self.captureRadius:GetValue() then
		self:EndChase()
		-- captured!
		self:PossessTargetFruit()
		return
	end
	
	local dir = vec:Normalised()
	
	position = position + dir * self.chaseSpeed:GetValue() * dt
	self:SetPosition(position)
end

function GhostAIComponent:CheckTargetFruit()
	if self.targetFruit then
		-- determine in this fruit is still valid
		if not self.targetFruit:IsReleased() and
		   not self.targetFruit:IsSliced() and
		   not self.targetFruit:FindEntity("GhostPossessedComponent") then
			return
		end
	end
	
	self.targetFruit = nil
	
	local availableFruit = {}
	
	local fruits = GameCore.FindEntitiesOfType("FruitContainer", false)
	for i, fruit in ipairs(fruits) do
		-- Not sliced, is thrown and not possessed
		if not fruit:IsSliced() and fruit:IsThrown() and not fruit:FindEntity("GhostPossessedComponent")then	
			-- Check if within the chase area
			if self:IsInChaseArea(fruit) then
				table.insert(availableFruit, fruit)
			end
		end
	end
	
	if #availableFruit == 0 then
		return
	end
	
	-- choose a random one
	local i = math.random(1, #availableFruit)
	self.targetFruit = availableFruit[i]
	
	-- play sound when new fruit target is found
	local chaseSound = GetGlobalSoundCue("ModifierDojoGhostbustersFloat")
	if not chaseSound then
		self:LogE("Missing sound ModifierDojoGhostbustersFloat!")
	end
	
	chaseSound:Play()
end

function GhostAIComponent:EndChase()

end

function GhostAIComponent:ResolvePadding()
	local screenWidth = DeviceInfo.GetScreenWidth()
	if screenWidth == 0 then
		self:LogE("Resolved screen width is 0!")
		return
	end
	
	local screenHeight = DeviceInfo.GetScreenHeight()
	if screenHeight == 0 then
		self:LogE("Resolved screen height is 0!")
		return
	end
	
	local padding = self.screenChasePadding:GetValue()
	
	if padding.x < (-screenWidth * 2) then
		self:LogE("Screen Chase Padding bigger than the screen width!")
		padding.x = screenWidth * 0.5
		
		-- will resolve again
		self.screenChasePadding:SetValue(padding)
		return
	end
	
	if padding.y < (-screenHeight * 2) then
		self:LogE("Screen Chase Padding bigger than the screen height!")
		padding.y = screenHeight * 0.5
		
		-- will resolve again
		self.screenChasePadding:SetValue(padding)
		return	
	end
	
	self.resolvedArea = Vector2(screenWidth + padding.x, screenHeight + padding.y)
end

function GhostAIComponent:IsInChaseArea(entity)
	if not entity or entity:IsReleased() then
		return false
	end
	
	local position = GetProp(entity, "position", "vector3")
	local halfWidth = self.resolvedArea.x * 0.5
	local halfHeight = self.resolvedArea.y * 0.5
	
	if position.x < -halfWidth then return false end
	if position.x > halfWidth then return false end
	if position.y < -halfHeight then return false end
	if position.y > halfHeight then return false end
	
	return true
end

function GhostAIComponent:PossessTargetFruit()
	if not self.targetFruit or self.targetFruit:IsReleased() then
		self.targetFruit = nil
		return
	end
	
	local possessSound = GetGlobalSoundCue("ModifierDojoGhostbustersPossess")
	if not possessSound then
		self:LogE("Missing sound ModifierDojoGhostbustersPossess!")
	else
		possessSound:Play()
	end
	
	-- Get OnPossess Actor
	local onPossessActor = self:FindEntity("parent.OnPossess")
	if not onPossessActor then
		self:LogV("No possession effects will occur.")
		-- Delete self
		self:DestroyParentActor()
		return
	end
	
	-- Attach all components
	local children = onPossessActor:GetChildren()
	
	for i, child in ipairs(children) do
		self.targetFruit:AddChild(child)
	end
	
	-- no need for this emitter anymore
	self.disappearEmitter = nil
	
	-- Delete self
	self:DestroyParentActor()
end