require("Helpers")

ScreenSpawnComponent = ScreenSpawnComponent or {}

function ScreenSpawnComponent:construct()
	self.padding = self:CreateProperty("Padding", "vector2", Vector2(100,100))
	self.spawnRoot = self:CreateProperty("SpawnRoot", "string", "")
	self.maxSpawned = self:CreateProperty("Max Spawned", "int", 3)
	self.frequency = self:CreateProperty("Frequency", "float", 3.0)
	
	self.resolvedArea = Vector2(0,0)
	self.timer = 0
end

function ScreenSpawnComponent:OnPropertyChanged(prop)
	if prop == self.padding then
		self:ResolveBounds()
	end
end

function ScreenSpawnComponent:FirstUpdate()
	self:ResolveBounds()
end

function ScreenSpawnComponent:OnEnable()
	self.timer = 0
end

function ScreenSpawnComponent:Update(dt)

	if Game.IsPaused() then
		return
	end
	
	local legacyDt = Game.GetLegacyDt()

	self.timer = self.timer + legacyDt
	if self.timer > self.frequency:GetValue() then
		if self:CanSpawn() then
			self:Spawn()
		end
		
		self.timer = 0
	end
end

function ScreenSpawnComponent:ResolveBounds()
	local screenWidth = DeviceInfo.GetScreenWidth()
	if screenWidth == 0 then
		self:LogE("Resolved screen width is 0!")
		return
	end
	
	local screenHeight = DeviceInfo.GetScreenHeight()
	if screenHeight == 0 then
		self:LogE("Resolved screen height is 0!")
		return
	end
	
	local padding = self.padding:GetValue()
	
	if padding.x < (-screenWidth * 2) then
		self:LogE("Padding bigger than the screen width!")
		padding.x = screenWidth * 0.5
		
		-- will resolve again
		self.padding:SetValue(padding)
		return
	end
	
	if padding.y < (-screenHeight * 2) then
		self:LogE("Padding bigger than the screen height!")
		padding.y = screenHeight * 0.5
		
		-- will resolve again
		self.padding:SetValue(padding)
		return	
	end
	
	self.resolvedArea = Vector2(screenWidth + padding.x, screenHeight + padding.y)
end

function ScreenSpawnComponent:Spawn()
	if IsNilOrEmpty(self.spawnRoot:GetValue()) then
		self:LogE("SpawnRoot is empty!")
		return	
	end
	
	local spawnRoot = self:FindEntity(self.spawnRoot:GetValue())
	if not spawnRoot then
		self:LogE("Could not find SpawnRoot at " .. self.spawnRoot:GetValue())
		return
	end
	
	local children = self:GetChildrenActors(true)
	
	for i, child in ipairs(children) do
		self:RandomlyInstantiate(child, spawnRoot)
	end
end

function ScreenSpawnComponent:RandomlyInstantiate(actor, spawnRoot)
	if not actor or not spawnRoot then
		self:LogE("Either actor or spawn root is nil!")
		return
	end
	
	-- get random position within bounds
	local x = (math.random() - 0.5) * self.resolvedArea.x
	local y = (math.random() - 0.5) * self.resolvedArea.y
	
	-- clone
	local cloned = actor:Clone()
	
	-- place under spawnRoot
	cloned:SetParent(spawnRoot)
	
	-- enable
	SetProp(cloned, "enabled", "bool", true)
	SetProp(cloned, "position", "vector3", Vector3(x, y, 0))
end

function ScreenSpawnComponent:CanSpawn()
	local spawnRoot = self:FindEntity(self.spawnRoot:GetValue())
	if not spawnRoot then
		self:LogE("Could not find SpawnRoot at " .. self.spawnRoot:GetValue())
		return false
	end
	
	local children = spawnRoot:GetChildren()

	-- assume all actor children were spawned
	local count = 0
	for i, child in ipairs(children) do
		if child:IsType("Actor") then
			count = count + 1
			if count >= self.maxSpawned:GetValue() then
				return false
			end
		end
	end
	
	return true
end